MODULE limitd_th
   !!======================================================================
   !!                       ***  MODULE limitd_th ***
   !!   LIM3 ice model : ice thickness distribution: Thermodynamics
   !!======================================================================
   !! History :   -   !          (W. H. Lipscomb and E.C. Hunke) CICE (c) original code
   !!            3.0  ! 2005-12  (M. Vancoppenolle) adaptation to LIM-3
   !!             -   ! 2006-06  (M. Vancoppenolle) adaptation to include salt, age
   !!             -   ! 2007-04  (M. Vancoppenolle) Mass conservation checked
   !!----------------------------------------------------------------------
#if defined key_lim3
   !!----------------------------------------------------------------------
   !!   'key_lim3' :                                   LIM3 sea-ice model
   !!----------------------------------------------------------------------
   !!   lim_itd_th_rem   :
   !!   lim_itd_th_reb   :
   !!   lim_itd_glinear  :
   !!   lim_itd_shiftice :
   !!----------------------------------------------------------------------
   USE par_oce          ! ocean parameters
   USE dom_oce          ! ocean domain
   USE phycst           ! physical constants (ocean directory) 
   USE thd_ice          ! LIM-3 thermodynamic variables
   USE ice              ! LIM-3 variables
   USE limvar           ! LIM-3 variables
   USE limcons          ! conservation tests
   !
   USE prtctl           ! Print control
   USE in_out_manager   ! I/O manager
   USE lib_mpp          ! MPP library
   USE wrk_nemo         ! work arrays
   USE lib_fortran      ! to use key_nosignedzero

   IMPLICIT NONE
   PRIVATE

   PUBLIC   lim_itd_th_rem
   PUBLIC   lim_itd_th_reb

   !!----------------------------------------------------------------------
   !! NEMO/LIM3 4.0 , UCL - NEMO Consortium (2010)
   !! $Id$
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE lim_itd_th_rem( kt )
      !!------------------------------------------------------------------
      !!                ***  ROUTINE lim_itd_th_rem ***
      !!
      !! ** Purpose :   computes the redistribution of ice thickness
      !!              after thermodynamic growth of ice thickness
      !!
      !! ** Method  : Linear remapping 
      !!
      !! References : W.H. Lipscomb, JGR 2001
      !!------------------------------------------------------------------
      INTEGER , INTENT (in) ::   kt      ! Ocean time step 
      !
      INTEGER  ::   ji, jj, jl     ! dummy loop index
      INTEGER  ::   ii, ij         ! 2D corresponding indices to ji
      INTEGER  ::   nd             ! local integer
      REAL(wp) ::   zx1, zwk1, zdh0, zetamin, zdamax   ! local scalars
      REAL(wp) ::   zx2, zwk2, zda0, zetamax           !   -      -
      REAL(wp) ::   zx3        

      INTEGER , POINTER, DIMENSION(:,:,:) ::   zdonor   ! donor category index

      REAL(wp), POINTER, DIMENSION(:,:,:) ::   zdhice      ! ice thickness increment
      REAL(wp), POINTER, DIMENSION(:,:,:) ::   g0          ! coefficients for fitting the line of the ITD
      REAL(wp), POINTER, DIMENSION(:,:,:) ::   g1          ! coefficients for fitting the line of the ITD
      REAL(wp), POINTER, DIMENSION(:,:,:) ::   hL          ! left boundary for the ITD for each thickness
      REAL(wp), POINTER, DIMENSION(:,:,:) ::   hR          ! left boundary for the ITD for each thickness
      REAL(wp), POINTER, DIMENSION(:,:,:) ::   zdaice, zdvice          ! local increment of ice area and volume
      INTEGER , DIMENSION(jpij)  ::   nind_i, nind_j          ! compressed indices for i/j directions
      REAL(wp)                            ::   zslope                  ! used to compute local thermodynamic "speeds"
      REAL(wp), POINTER, DIMENSION(:,:)   ::   zhb0, zhb1              ! category boundaries for thinnes categories
      INTEGER , POINTER, DIMENSION(:,:)   ::   zremap_flag      ! compute remapping or not ????
      REAL(wp), POINTER, DIMENSION(:,:,:) ::   zhbnew           ! new boundaries of ice categories
      !!------------------------------------------------------------------

      CALL wrk_alloc( jpi,jpj, zremap_flag )
      CALL wrk_alloc( jpi,jpj,jpl-1, zdonor )
      CALL wrk_alloc( jpi,jpj,jpl, zdhice, g0, g1, hL, hR )
      CALL wrk_alloc( jpi,jpj,jpl-1, zdaice, zdvice )   
      CALL wrk_alloc( jpi,jpj,jpl+1, zhbnew, kkstart = 0 )   
      CALL wrk_alloc( jpi,jpj, zhb0, zhb1 )

      IF( kt == nit000 .AND. lwp) THEN
         WRITE(numout,*)
         WRITE(numout,*) 'lim_itd_th_rem  : Remapping the ice thickness distribution'
         WRITE(numout,*) '~~~~~~~~~~~~~~~'
      ENDIF

      !-----------------------------------------------------------------------------------------------
      !  3) Identify grid cells with ice
      !-----------------------------------------------------------------------------------------------
      nidx = 0 ; idxice(:) = 0
      DO jj = 1, jpj
         DO ji = 1, jpi
            IF ( at_i(ji,jj) > epsi10 ) THEN
               nidx         = nidx + 1
               nind_i(nidx) = ji
               nind_j(nidx) = jj
               idxice( nidx ) = (jj - 1) * jpi + ji
               zremap_flag(ji,jj) = 1
            ELSE
               zremap_flag(ji,jj) = 0
            ENDIF
         END DO
      END DO

      !-----------------------------------------------------------------------------------------------
      !  4) Compute new category boundaries: 1:jpl-1
      !-----------------------------------------------------------------------------------------------
      zdhice(:,:,:) = 0._wp
      zhbnew(:,:,:) = 0._wp
      zhb0(:,:) = hi_max(0)
      zhb1(:,:) = hi_max(1)

      IF( nidx > 0 ) THEN

         ! Compute thickness change in each ice category
         DO jl = 1, jpl
            DO ji = 1, nidx
               ii = nind_i(ji)
               ij = nind_j(ji)
               zdhice(ii,ij,jl) = ht_i(ii,ij,jl) - ht_i_b(ii,ij,jl)
            END DO
         END DO
         
         DO jl = 1, jpl - 1
            DO ji = 1, nidx
               ii = nind_i(ji)
               ij = nind_j(ji)
               !
               ! --- New boundary categories Hn* = Hn + Fn*dt --- !
               !     Fn*dt = ( fn + (fn+1 - fn)/(hn+1 - hn) * (Hn - hn) ) * dt = zdhice + zslope * (Hmax - ht_i_b)
               !
               IF    ( a_i_b(ii,ij,jl) >  epsi10 .AND. a_i_b(ii,ij,jl+1) >  epsi10 ) THEN   ! a_i_b(jl+1) & a_i_b(jl) /= 0
                  zslope           = ( zdhice(ii,ij,jl+1) - zdhice(ii,ij,jl) ) / ( ht_i_b(ii,ij,jl+1) - ht_i_b(ii,ij,jl) )
                  zhbnew(ii,ij,jl) = hi_max(jl) + zdhice(ii,ij,jl) + zslope * ( hi_max(jl) - ht_i_b(ii,ij,jl) )
               ELSEIF( a_i_b(ii,ij,jl) >  epsi10 .AND. a_i_b(ii,ij,jl+1) <= epsi10 ) THEN   ! a_i_b(jl+1)=0 => Hn* = Hn + fn*dt
                  zhbnew(ii,ij,jl) = hi_max(jl) + zdhice(ii,ij,jl)
               ELSEIF( a_i_b(ii,ij,jl) <= epsi10 .AND. a_i_b(ii,ij,jl+1) >  epsi10 ) THEN   ! a_i_b(jl)=0 => Hn* = Hn + fn+1*dt
                  zhbnew(ii,ij,jl) = hi_max(jl) + zdhice(ii,ij,jl+1)
               ELSE                                                                         ! a_i_b(jl+1) & a_i_b(jl) = 0 
                  zhbnew(ii,ij,jl) = hi_max(jl)
               ENDIF
            
               ! --- 2 conditions for remapping --- !
               ! 1) hn(t+1)+espi < Hn* < hn+1(t+1)-epsi               
               !    Note: hn(t+1) must not be too close to either HR or HL otherwise a division by nearly 0 is possible 
               !          in lim_itd_glinear in the case (HR-HL) = 3(Hice - HL) or = 3(HR - Hice)
               IF( a_i(ii,ij,jl  ) > epsi10 .AND. ht_i(ii,ij,jl  ) > ( zhbnew(ii,ij,jl) - epsi10 ) )   zremap_flag(ii,ij) = 0
               IF( a_i(ii,ij,jl+1) > epsi10 .AND. ht_i(ii,ij,jl+1) < ( zhbnew(ii,ij,jl) + epsi10 ) )   zremap_flag(ii,ij) = 0
               
               ! 2) Hn-1 < Hn* < Hn+1  
               IF( zhbnew(ii,ij,jl) < hi_max(jl-1) )   zremap_flag(ii,ij) = 0
               IF( zhbnew(ii,ij,jl) > hi_max(jl+1) )   zremap_flag(ii,ij) = 0
               
            END DO
         END DO
         
         ! --- New boundary for category jpl --- !
         DO ji = 1, nidx
            ii = nind_i(ji)
            ij = nind_j(ji)
            IF( a_i(ii,ij,jpl) > epsi10 ) THEN
               zhbnew(ii,ij,jpl) = MAX( hi_max(jpl-1), 3._wp * ht_i(ii,ij,jpl) - 2._wp * zhbnew(ii,ij,jpl-1) )
            ELSE
               zhbnew(ii,ij,jpl) = hi_max(jpl)  
            ENDIF
            
            ! 1 condition for remapping for the 1st category
            ! H0+epsi < h1(t) < H1-epsi 
            !    h1(t) must not be too close to either HR or HL otherwise a division by nearly 0 is possible 
            !    in lim_itd_glinear in the case (HR-HL) = 3(Hice - HL) or = 3(HR - Hice)
            IF( ht_i_b(ii,ij,1) < ( hi_max(0) + epsi10 ) )   zremap_flag(ii,ij) = 0
            IF( ht_i_b(ii,ij,1) > ( hi_max(1) - epsi10 ) )   zremap_flag(ii,ij) = 0
         END DO
         
      ENDIF
   
      !-----------------------------------------------------------------------------------------------
      !  5) Identify cells where ITD is to be remapped
      !-----------------------------------------------------------------------------------------------
      nidx = 0
      DO jj = 1, jpj
         DO ji = 1, jpi
            IF( zremap_flag(ji,jj) == 1 ) THEN
               nidx         = nidx + 1
               nind_i(nidx) = ji
               nind_j(nidx) = jj
            ENDIF
         END DO 
      END DO 

      !-----------------------------------------------------------------------------------------------
      !  7) Compute g(h) 
      !-----------------------------------------------------------------------------------------------
      IF( nidx > 0 ) THEN

         !- 7.1 g(h) for category 1 at start of time step
         CALL lim_itd_glinear( 1, zhb0, zhb1, ht_i_b(:,:,1), g0(:,:,1), g1(:,:,1), hL(:,:,1),   &
            &                  hR(:,:,1), zremap_flag )
         
         !- 7.2 Area lost due to melting of thin ice (first category,  1)
         DO ji = 1, nidx
            ii = nind_i(ji) 
            ij = nind_j(ji) 
            
            IF( a_i(ii,ij,1) > epsi10 ) THEN
               
               zdh0 = zdhice(ii,ij,1) !decrease of ice thickness in the lower category
               
               IF( zdh0 < 0.0 ) THEN      !remove area from category 1
                  zdh0 = MIN( -zdh0, hi_max(1) )
                  !Integrate g(1) from 0 to dh0 to estimate area melted
                  zetamax = MIN( zdh0, hR(ii,ij,1) ) - hL(ii,ij,1)
                  
                  IF( zetamax > 0.0 ) THEN
                     zx1    = zetamax
                     zx2    = 0.5 * zetamax * zetamax 
                     zda0   = g1(ii,ij,1) * zx2 + g0(ii,ij,1) * zx1                        ! ice area removed
                     zdamax = a_i(ii,ij,1) * (1.0 - ht_i(ii,ij,1) / ht_i_b(ii,ij,1) ) ! Constrain new thickness <= ht_i                
                     zda0   = MIN( zda0, zdamax )                                                  ! ice area lost due to melting 
                     !     of thin ice (zdamax > 0)
                     ! Remove area, conserving volume
                     ht_i(ii,ij,1) = ht_i(ii,ij,1) * a_i(ii,ij,1) / ( a_i(ii,ij,1) - zda0 )
                     a_i(ii,ij,1)  = a_i(ii,ij,1) - zda0
                     v_i(ii,ij,1)  = a_i(ii,ij,1) * ht_i(ii,ij,1) ! clem-useless ?
                  ENDIF
                  
               ELSE ! if ice accretion zdh0 > 0
                  ! zhbnew was 0, and is shifted to the right to account for thin ice growth in openwater (F0 = f1)
                  zhbnew(ii,ij,0) = MIN( zdh0, hi_max(1) ) 
               ENDIF
               
            ENDIF
            
         END DO
         
         !- 7.3 g(h) for each thickness category  
         DO jl = 1, jpl
            CALL lim_itd_glinear( jl, zhbnew(:,:,jl-1), zhbnew(:,:,jl), ht_i(:,:,jl),  &
               &                  g0(:,:,jl), g1(:,:,jl), hL(:,:,jl), hR(:,:,jl), zremap_flag )
         END DO
         
         !-----------------------------------------------------------------------------------------------
         !  8) Compute area and volume to be shifted across each boundary (Eq. 18)
         !-----------------------------------------------------------------------------------------------
         
         DO jl = 1, jpl - 1
            DO jj = 1, jpj
               DO ji = 1, jpi
                  zdonor(ji,jj,jl) = 0
                  zdaice(ji,jj,jl) = 0.0
                  zdvice(ji,jj,jl) = 0.0
               END DO
            END DO
            
            DO ji = 1, nidx
               ii = nind_i(ji)
               ij = nind_j(ji)
               
               ! left and right integration limits in eta space
               IF (zhbnew(ii,ij,jl) > hi_max(jl)) THEN ! Hn* > Hn => transfer from jl to jl+1
                  zetamin = MAX( hi_max(jl), hL(ii,ij,jl) ) - hL(ii,ij,jl)       ! hi_max(jl) - hL 
                  zetamax = MIN( zhbnew(ii,ij,jl), hR(ii,ij,jl) ) - hL(ii,ij,jl) ! hR - hL
                  zdonor(ii,ij,jl) = jl
               ELSE                                    ! Hn* <= Hn => transfer from jl+1 to jl
                  zetamin = 0.0
                  zetamax = MIN( hi_max(jl), hR(ii,ij,jl+1) ) - hL(ii,ij,jl+1)   ! hi_max(jl) - hL
                  zdonor(ii,ij,jl) = jl + 1
               ENDIF
               zetamax = MAX( zetamax, zetamin ) ! no transfer if etamax < etamin
               
               zx1  = zetamax - zetamin
               zwk1 = zetamin * zetamin
               zwk2 = zetamax * zetamax
               zx2  = 0.5 * ( zwk2 - zwk1 )
               zwk1 = zwk1 * zetamin
               zwk2 = zwk2 * zetamax
               zx3  = 1.0 / 3.0 * ( zwk2 - zwk1 )
               nd   = zdonor(ii,ij,jl)
               zdaice(ii,ij,jl) = g1(ii,ij,nd)*zx2 + g0(ii,ij,nd)*zx1
               zdvice(ii,ij,jl) = g1(ii,ij,nd)*zx3 + g0(ii,ij,nd)*zx2 + zdaice(ii,ij,jl)*hL(ii,ij,nd)
               
            END DO
         END DO
         
         !!----------------------------------------------------------------------------------------------
         !! 9) Shift ice between categories
         !!----------------------------------------------------------------------------------------------
         CALL lim_itd_shiftice ( zdonor, zdaice, zdvice )
         
         !!----------------------------------------------------------------------------------------------
         !! 10) Make sure ht_i >= minimum ice thickness hi_min
         !!----------------------------------------------------------------------------------------------
         
         DO ji = 1, nidx
            ii = nind_i(ji)
            ij = nind_j(ji)
            IF ( a_i(ii,ij,1) > epsi10 .AND. ht_i(ii,ij,1) < rn_himin ) THEN
               a_i (ii,ij,1) = a_i(ii,ij,1) * ht_i(ii,ij,1) / rn_himin 
               ! MV MP 2016
               IF ( nn_pnd_scheme > 0 ) THEN
                  a_ip(ii,ij,1) = a_ip(ii,ij,1) * ht_i(ii,ij,1) / rn_himin
               ENDIF
               ! END MV MP 2016
               ht_i(ii,ij,1) = rn_himin
            ENDIF
         END DO
         
      ENDIF

      CALL wrk_dealloc( jpi,jpj, zremap_flag )
      CALL wrk_dealloc( jpi,jpj,jpl-1, zdonor )
      CALL wrk_dealloc( jpi,jpj,jpl, zdhice, g0, g1, hL, hR )
      CALL wrk_dealloc( jpi,jpj,jpl-1, zdaice, zdvice )   
      CALL wrk_dealloc( jpi,jpj,jpl+1, zhbnew, kkstart = 0 )   
      CALL wrk_dealloc( jpi,jpj, zhb0, zhb1 )

   END SUBROUTINE lim_itd_th_rem


   SUBROUTINE lim_itd_glinear( num_cat, HbL, Hbr, hice, g0, g1, hL, hR, zremap_flag )
      !!------------------------------------------------------------------
      !!                ***  ROUTINE lim_itd_glinear ***
      !!
      !! ** Purpose :   build g(h) satisfying area and volume constraints (Eq. 6 and 9)
      !!
      !! ** Method  :   g(h) is linear and written as: g(eta) = g1(eta) + g0
      !!                with eta = h - HL
      !!                
      !!------------------------------------------------------------------
      INTEGER                     , INTENT(in   ) ::   num_cat      ! category index
      REAL(wp), DIMENSION(jpi,jpj), INTENT(in   ) ::   HbL, HbR     ! left and right category boundaries
      REAL(wp), DIMENSION(jpi,jpj), INTENT(in   ) ::   hice         ! ice thickness
      REAL(wp), DIMENSION(jpi,jpj), INTENT(  out) ::   g0, g1       ! coefficients in linear equation for g(eta)
      REAL(wp), DIMENSION(jpi,jpj), INTENT(  out) ::   hL           ! min value of range over which g(h) > 0
      REAL(wp), DIMENSION(jpi,jpj), INTENT(  out) ::   hR           ! max value of range over which g(h) > 0
      INTEGER , DIMENSION(jpi,jpj), INTENT(in   ) ::   zremap_flag  !
      !
      INTEGER  ::   ji,jj        ! horizontal indices
      REAL(wp) ::   zh13         ! HbL + 1/3 * (HbR - HbL)
      REAL(wp) ::   zh23         ! HbL + 2/3 * (HbR - HbL)
      REAL(wp) ::   zdhr         ! 1 / (hR - hL)
      REAL(wp) ::   zwk1, zwk2   ! temporary variables
      !!------------------------------------------------------------------
      !
      DO jj = 1, jpj
         DO ji = 1, jpi
            !
            IF( zremap_flag(ji,jj) == 1 .AND. a_i(ji,jj,num_cat) > epsi10   &
               &                        .AND. hice(ji,jj)        > 0._wp )  THEN

               ! Initialize hL and hR
               hL(ji,jj) = HbL(ji,jj)
               hR(ji,jj) = HbR(ji,jj)

               ! Change hL or hR if hice falls outside central third of range,
               ! so that hice is in the central third of the range [HL HR]
               zh13 = 1.0 / 3.0 * ( 2.0 * hL(ji,jj) +       hR(ji,jj) )
               zh23 = 1.0 / 3.0 * (       hL(ji,jj) + 2.0 * hR(ji,jj) )

               IF    ( hice(ji,jj) < zh13 ) THEN   ;   hR(ji,jj) = 3._wp * hice(ji,jj) - 2._wp * hL(ji,jj) ! move HR to the left
               ELSEIF( hice(ji,jj) > zh23 ) THEN   ;   hL(ji,jj) = 3._wp * hice(ji,jj) - 2._wp * hR(ji,jj) ! move HL to the right
               ENDIF

               ! Compute coefficients of g(eta) = g0 + g1*eta
               zdhr = 1._wp / (hR(ji,jj) - hL(ji,jj))
               zwk1 = 6._wp * a_i(ji,jj,num_cat) * zdhr
               zwk2 = ( hice(ji,jj) - hL(ji,jj) ) * zdhr
               g0(ji,jj) = zwk1 * ( 2._wp / 3._wp - zwk2 )      ! Eq. 14
               g1(ji,jj) = 2._wp * zdhr * zwk1 * ( zwk2 - 0.5 ) ! Eq. 14
               !
            ELSE  ! remap_flag = .false. or a_i < epsi10 
               hL(ji,jj) = 0._wp
               hR(ji,jj) = 0._wp
               g0(ji,jj) = 0._wp
               g1(ji,jj) = 0._wp
            ENDIF
            !
         END DO
      END DO
      !
   END SUBROUTINE lim_itd_glinear


   SUBROUTINE lim_itd_shiftice( zdonor, zdaice, zdvice )
      !!------------------------------------------------------------------
      !!                ***  ROUTINE lim_itd_shiftice ***
      !!
      !! ** Purpose :   shift ice across category boundaries, conserving everything
      !!              ( area, volume, energy, age*vol, and mass of salt )
      !!
      !! ** Method  :
      !!------------------------------------------------------------------
      INTEGER , DIMENSION(jpi,jpj,jpl-1), INTENT(in   ) ::   zdonor   ! donor category index
      REAL(wp), DIMENSION(jpi,jpj,jpl-1), INTENT(inout) ::   zdaice   ! ice area transferred across boundary
      REAL(wp), DIMENSION(jpi,jpj,jpl-1), INTENT(inout) ::   zdvice   ! ice volume transferred across boundary

      INTEGER ::   ji, jj, jl, jl2, jl1, jk   ! dummy loop indices

      REAL(wp), DIMENSION(jpi,jpj,jpl) ::   zaTsfn
      REAL(wp), DIMENSION(jpi,jpj)     ::   zworka            ! temporary array used here

      REAL(wp) ::   ztrans             ! ice/snow transferred
      !!------------------------------------------------------------------

      !----------------------------------------------------------------------------------------------
      ! 1) Define a variable equal to a_i*T_su
      !----------------------------------------------------------------------------------------------
      DO jl = 1, jpl
         DO jj = 1, jpj
            DO ji = 1, jpi
               zaTsfn(ji,jj,jl) = a_i(ji,jj,jl) * t_su(ji,jj,jl)
            END DO
         END DO
      END DO
      
      !-------------------------------------------------------------------------------
      ! 2) Transfer volume and energy between categories
      !-------------------------------------------------------------------------------
      DO jl = 1, jpl - 1
         DO jj = 1, jpj
            DO ji = 1, jpi

               jl1 = zdonor(ji,jj,jl)
               rswitch       = MAX( 0._wp , SIGN( 1._wp , v_i(ji,jj,jl1) - epsi10 ) )
               zworka(ji,jj) = zdvice(ji,jj,jl) / MAX( v_i(ji,jj,jl1), epsi10 ) * rswitch

               IF( jl1 == jl) THEN   ;   jl2 = jl1+1
               ELSE                  ;   jl2 = jl 
               ENDIF
               
               ! Ice areas
               a_i(ji,jj,jl1) = a_i(ji,jj,jl1) - zdaice(ji,jj,jl)
               a_i(ji,jj,jl2) = a_i(ji,jj,jl2) + zdaice(ji,jj,jl)
               
               ! Ice volumes
               v_i(ji,jj,jl1) = v_i(ji,jj,jl1) - zdvice(ji,jj,jl) 
               v_i(ji,jj,jl2) = v_i(ji,jj,jl2) + zdvice(ji,jj,jl)
               
               ! Snow volumes
               ztrans         = v_s(ji,jj,jl1) * zworka(ji,jj)
               v_s(ji,jj,jl1) = v_s(ji,jj,jl1) - ztrans
               v_s(ji,jj,jl2) = v_s(ji,jj,jl2) + ztrans 
               
               ! Snow heat content  
               ztrans             = e_s(ji,jj,1,jl1) * zworka(ji,jj)
               e_s(ji,jj,1,jl1)   = e_s(ji,jj,1,jl1) - ztrans
               e_s(ji,jj,1,jl2)   = e_s(ji,jj,1,jl2) + ztrans
               
               ! Ice age 
               ztrans             = oa_i(ji,jj,jl1) * zdaice(ji,jj,jl)
               oa_i(ji,jj,jl1)    = oa_i(ji,jj,jl1) - ztrans
               oa_i(ji,jj,jl2)    = oa_i(ji,jj,jl2) + ztrans
               
               ! Ice salinity
               ztrans             = smv_i(ji,jj,jl1) * zworka(ji,jj)
               smv_i(ji,jj,jl1)   = smv_i(ji,jj,jl1) - ztrans
               smv_i(ji,jj,jl2)   = smv_i(ji,jj,jl2) + ztrans
               
               ! Surface temperature
               ztrans             = t_su(ji,jj,jl1) * zdaice(ji,jj,jl)
               zaTsfn(ji,jj,jl1)  = zaTsfn(ji,jj,jl1) - ztrans
               zaTsfn(ji,jj,jl2)  = zaTsfn(ji,jj,jl2) + ztrans
               
               ! MV MP 2016 
               IF ( nn_pnd_scheme > 0 ) THEN
                  ! Pond fraction
                  ztrans          = a_ip(ji,jj,jl1) * zdaice(ji,jj,jl)
                  a_ip(ji,jj,jl1) = a_ip(ji,jj,jl1) - ztrans
                  a_ip(ji,jj,jl2) = a_ip(ji,jj,jl2) + ztrans
                  
                  ! Pond volume
                  ztrans          = v_ip(ji,jj,jl1) * zdaice(ji,jj,jl)
                  v_ip(ji,jj,jl1) = v_ip(ji,jj,jl1) - ztrans
                  v_ip(ji,jj,jl2) = v_ip(ji,jj,jl2) + ztrans
               ENDIF
               ! END MV MP 2016 
               
            END DO
         END DO
         
         ! Ice heat content
         DO jk = 1, nlay_i
            DO jj = 1, jpj
               DO ji = 1, jpi
                  
                  jl1 = zdonor(ji,jj,jl)
                  IF(jl1 == jl) THEN  ;  jl2 = jl+1
                  ELSE                ;  jl2 = jl
                  ENDIF
                  
                  ztrans            = e_i(ji,jj,jk,jl1) * zworka(ji,jj)
                  e_i(ji,jj,jk,jl1) = e_i(ji,jj,jk,jl1) - ztrans
                  e_i(ji,jj,jk,jl2) = e_i(ji,jj,jk,jl2) + ztrans 
               END DO
            END DO
         END DO
         
      END DO                   ! boundaries, 1 to jpl-1
      
      ! Update ice thickness and temperature
      DO jl = 1, jpl
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF ( a_i(ji,jj,jl) > epsi10 ) THEN 
                  ht_i(ji,jj,jl)  =  v_i   (ji,jj,jl) / a_i(ji,jj,jl) 
                  t_su(ji,jj,jl)  =  zaTsfn(ji,jj,jl) / a_i(ji,jj,jl) 
               ELSE
                  ht_i(ji,jj,jl)  = 0._wp
                  t_su(ji,jj,jl)  = rt0
               ENDIF
            END DO
         END DO
      END DO      
      !
   END SUBROUTINE lim_itd_shiftice
   

   SUBROUTINE lim_itd_th_reb
      !!------------------------------------------------------------------
      !!                ***  ROUTINE lim_itd_th_reb ***
      !!
      !! ** Purpose : rebin - rebins thicknesses into defined categories
      !!
      !! ** Method  :
      !!------------------------------------------------------------------
      INTEGER ::   ji, jj, jl   ! dummy loop indices
      INTEGER ::   zshiftflag          ! = .true. if ice must be shifted

      INTEGER , DIMENSION(jpi,jpj,jpl) ::   zdonor           ! donor category index
      REAL(wp), DIMENSION(jpi,jpj,jpl) ::   zdaice, zdvice   ! ice area and volume transferred
      !!------------------------------------------------------------------
      
      !------------------------------------------------------------------------------
      ! 1) Compute ice thickness.
      !------------------------------------------------------------------------------
      DO jl = 1, jpl
         DO jj = 1, jpj
            DO ji = 1, jpi 
               rswitch = MAX( 0._wp , SIGN( 1._wp, a_i(ji,jj,jl) - epsi10 ) )
               ht_i(ji,jj,jl) = v_i (ji,jj,jl) / MAX( a_i(ji,jj,jl) , epsi10 ) * rswitch
            END DO
         END DO
      END DO

      !------------------------------------------------------------------------------
      ! 2) If a category thickness is not in bounds, shift the
      ! entire area, volume, and energy to the neighboring category
      !------------------------------------------------------------------------------
      !-------------------------
      ! Initialize shift arrays
      !-------------------------
      DO jl = 1, jpl
         zdonor(:,:,jl) = 0
         zdaice(:,:,jl) = 0._wp
         zdvice(:,:,jl) = 0._wp
      END DO

      !-------------------------
      ! Move thin categories up
      !-------------------------

      DO jl = 1, jpl - 1  ! loop over category boundaries

         !---------------------------------------
         ! identify thicknesses that are too big
         !---------------------------------------
         zshiftflag = 0

         DO jj = 1, jpj 
            DO ji = 1, jpi 
               IF( a_i(ji,jj,jl) > epsi10 .AND. ht_i(ji,jj,jl) > hi_max(jl) ) THEN 
                  zshiftflag        = 1
                  zdonor(ji,jj,jl)  = jl 
                  ! clem: how much of a_i you send in cat sup is somewhat arbitrary
                  zdaice(ji,jj,jl)  = a_i(ji,jj,jl) * ( ht_i(ji,jj,jl) - hi_max(jl) + epsi20 ) / ht_i(ji,jj,jl)  
                  zdvice(ji,jj,jl)  = v_i(ji,jj,jl) - ( a_i(ji,jj,jl) - zdaice(ji,jj,jl) ) * ( hi_max(jl) - epsi20 )
               ENDIF
            END DO
         END DO
         IF(lk_mpp)   CALL mpp_max( zshiftflag ) ! clem: for reproducibility ???

         IF( zshiftflag == 1 ) THEN            ! Shift ice between categories
            CALL lim_itd_shiftice( zdonor, zdaice, zdvice )
            ! Reset shift parameters
            zdonor(:,:,jl) = 0
            zdaice(:,:,jl) = 0._wp
            zdvice(:,:,jl) = 0._wp
         ENDIF
         !
      END DO

      !----------------------------
      ! Move thick categories down
      !----------------------------

      DO jl = jpl - 1, 1, -1       ! loop over category boundaries

         !-----------------------------------------
         ! Identify thicknesses that are too small
         !-----------------------------------------
         zshiftflag = 0

         DO jj = 1, jpj
            DO ji = 1, jpi
               IF( a_i(ji,jj,jl+1) > epsi10 .AND. ht_i(ji,jj,jl+1) <= hi_max(jl) ) THEN
                  !
                  zshiftflag = 1
                  zdonor(ji,jj,jl) = jl + 1
                  zdaice(ji,jj,jl) = a_i(ji,jj,jl+1) 
                  zdvice(ji,jj,jl) = v_i(ji,jj,jl+1)
               ENDIF
            END DO
         END DO

         IF(lk_mpp)   CALL mpp_max( zshiftflag ) ! clem: for reproducibility ???
         
         IF( zshiftflag == 1 ) THEN            ! Shift ice between categories
            CALL lim_itd_shiftice( zdonor, zdaice, zdvice )
            ! Reset shift parameters
            zdonor(:,:,jl) = 0
            zdaice(:,:,jl) = 0._wp
            zdvice(:,:,jl) = 0._wp
         ENDIF

      END DO
      !
   END SUBROUTINE lim_itd_th_reb

#else
   !!----------------------------------------------------------------------
   !!   Default option            Dummy module         NO LIM sea-ice model
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE lim_itd_th_rem
   END SUBROUTINE lim_itd_th_rem
   SUBROUTINE lim_itd_glinear
   END SUBROUTINE lim_itd_glinear
   SUBROUTINE lim_itd_shiftice
   END SUBROUTINE lim_itd_shiftice
   SUBROUTINE lim_itd_th_reb
   END SUBROUTINE lim_itd_th_reb
#endif
   !!======================================================================
END MODULE limitd_th
